import { isArrayLike } from "@thi.ng/checks/is-arraylike";
import { isNumber } from "@thi.ng/checks/is-number";
import { isPlainObject } from "@thi.ng/checks/is-plain-object";
const RE_VEC = /^[iub]?vec[234]$/;
const RE_MAT = /^mat[234]$/;
/**
 * Returns true if given `t` is a {@link Term}-like object.
 *
 * @param t -
 */
export const isTerm = (t) => isPlainObject(t) && !!t.tag && !!t.type;
/**
 * Returns true, if given term evaluates to a boolean value.
 */
export const isBool = (t) => t.type === "bool";
/**
 * Returns true, if given term evaluates to a float value.
 */
export const isFloat = (t) => t.type === "float";
/**
 * Returns true, if given term evaluates to a signed integer value.
 */
export const isInt = (t) => t.type === "int";
/**
 * Returns true, if given term evaluates to an unsigned integer value.
 */
export const isUint = (t) => t.type === "uint";
/**
 * Returns true, if given term is a literal.
 */
export const isLit = (t) => t.tag === "lit";
/**
 * Returns true, if given term is a float literal.
 */
export const isLitFloat = (t) => isLit(t) && isFloat(t);
/**
 * Returns true, if given term is a signed integer literal.
 */
export const isLitInt = (t) => isLit(t) && isInt(t);
/**
 * Returns true, if given term is a numeric literal (float, int, uint).
 */
export const isLitNumeric = (t) => isLit(t) && (isFloat(t) || isInt(t) || isUint(t));
/**
 * Returns true if t is a numeric literal with a JS number as value (not an
 * expression).
 *
 * @param t -
 */
export const isLitNumericConst = (t) => isLit(t) && isNumber(t.val);
/**
 * Returns true if t is a vector literal with a JS array as value (not an
 * expression).
 *
 * @param t -
 */
export const isLitVecConst = (t) => isLit(t) && isVec(t) && isArrayLike(t.val);
/**
 * Returns true, if given term evaluates to a vector value (vec, ivec, bvec).
 */
export const isVec = (t) => RE_VEC.test(t.type);
/**
 * Returns true, if given term evaluates to a matrix value.
 */
export const isMat = (t) => RE_MAT.test(t.type);
