import { isString } from "@thi.ng/checks/is-string";
import { assert } from "@thi.ng/errors/assert";
import { illegalArgs } from "@thi.ng/errors/illegal-arguments";
import { gensym } from "./idgen.js";
export function sym(type, ...xs) {
    let id;
    let opts;
    let init;
    switch (xs.length) {
        case 0:
            if (!isString(type)) {
                init = type;
                type = init.type;
            }
            break;
        case 1:
            if (isString(xs[0])) {
                id = xs[0];
            }
            else if (xs[0].tag) {
                init = xs[0];
            }
            else {
                opts = xs[0];
            }
            break;
        case 2:
            if (isString(xs[0])) {
                [id, opts] = xs;
            }
            else {
                [opts, init] = xs;
            }
            break;
        case 3:
            [id, opts, init] = xs;
            break;
        default:
            illegalArgs();
    }
    return {
        tag: "sym",
        type,
        id: id || gensym(),
        opts: opts || {},
        init: init,
    };
}
export const constSym = (type, id, opts, init) => sym(type, id || gensym(), { const: true, ...opts }, init);
/**
 * Defines a new symbol with optional initial array values.
 *
 * Important: Array initializers are UNSUPPORTED in GLSL ES v1 (WebGL),
 * any code using such initializers will only work under WebGL2 or other
 * targets.
 */
export const arraySym = (type, id, opts = {}, init) => {
    if (init && opts.num == null) {
        opts.num = init.length;
    }
    assert(opts.num != null, "missing array length");
    init &&
        assert(opts.num === init.length, `expected ${opts.num} items in array, but got ${init.length}`);
    const atype = (type + "[]");
    return {
        tag: "sym",
        type: atype,
        id: id || gensym(),
        opts,
        init: init
            ? {
                tag: "array_init",
                type: atype,
                init,
            }
            : undefined,
    };
};
export const input = (type, id, opts) => sym(type, id, { q: "in", type: "in", ...opts });
export const output = (type, id, opts) => sym(type, id, { q: "out", type: "out", ...opts });
export const uniform = (type, id, opts) => sym(type, id, { q: "in", type: "uni", ...opts });
